# Read a single benchmark file and return it's content as data frame.
# Columns: Attempt, Distance, Time, TTime.
read_benchmark <- function(file) {
	table_data <- read.table(file, header = TRUE, sep = "", quote = "", row.names = 1)
	return(table_data)
}

# Setup possible benchmark configurations - list of (nodeCount,edgeCount,depth)
bconfs <- list(
		list(1000, 500, 8), 
		list(1000, 500, 32), 
		list(1000, 2000, 4), 
		list(1000, 2000, 16),
		list(1000, 25000, 4),
		list(500, -1, 4),
		list(5000, 500, 16),
		list(5000, 1000, 32)
		)

# Possible sites (L - local, R - remote)
sites <- list('L', 'R')

# Go through all benchmark configurations and create a plot for each.
for(i in 1:length(bconfs)) {

	# Construct file names with data for each benchmark configuration (store in fnames)
	bc = bconfs[i]
	fname_base = paste(unlist(bc), collapse = "_")
	fnames <- list()
	for(j in 1:length(sites)) {
		for(k in 1:length(sites)) {
			fname = paste("data/", paste(fname_base, sites[j], sites[k], sep = "_"), sep = "")
			fnames <- c(fnames, fname)
		}
	}
	
	# Prepare plot
	max_time = 0
	max_dist = 0
	plot_colours <- rainbow(4)
	png(paste("data/", fname_base, sep = ""), width=800, units="px")
	
	bd = list()
	
	# Read each file name and add it to the plot
	for(j in 1:length(fnames)) {
	
		# Read file and order by Distance column
		benchmark_data = read_benchmark(fnames[[j]])
		bd[[j]] = benchmark_data
		
		# Update largest time and distance value
		max_time_t = max(benchmark_data$Time, benchmark_data$TTime)
		max_time = max(max_time, max_time_t)
		
		max_dist_t = max(benchmark_data$Distance)
		max_dist = max(max_dist, max_dist_t)
		
		# Draw line for getNeighbours
		#lines(benchmark_data$Time, type="o", pch=22, col=plot_colours[1])
	}
	
	# Start creating plot
	plot(bd[[1]]$Distance, bd[[1]]$Time, xlim=c(-1, max_dist), ylim=c(1, max_time), type="p", pch=1, col=plot_colours[1], ann=FALSE, log="y")
	lines(bd[[1]]$Distance, bd[[1]]$TTime, type="p", pch=5, col=plot_colours[[1]])
	
	# Draw rest of the lines
	for(j in 2:length(bd)) {
		lines(bd[[j]]$Distance, bd[[j]]$Time, type="p", pch=1, col=plot_colours[[j]])
		lines(bd[[j]]$Distance, bd[[j]]$TTime, type="p", pch=5, col=plot_colours[[j]])
	}
	
	# Create box around plot
	box()
	
	# Titles for axes
	title(main = paste( "Benchmark", fname_base ))
	title(xlab = "Node distance in graph")
	title(ylab = "Time [usec]")
	
	# Plot legend
	lrconfigs <- c( "LL", "LR", "RL", "RR" )
	gnconfigs <- c( "normal", "transitive" )
	aconfigs_grid <- expand.grid(lrconfigs,gnconfigs) 
	aconfigs <- apply(aconfigs_grid,1,function(x) paste(x[1],x[2],sep=" - ")) 
	legend("topright", max_time, aconfigs, cex = 0.9, pch = rep(c(1, 5), each=4), col = plot_colours)
	
	# Additional plot description
	library(grid)
	grid.text("Notes: Benchmark nodeCount_edgeCount_depth\nDistance -1 means infinity.\nLR - local node, remote searcher",
           x=unit(2, "mm"), y=unit(1, "npc") - unit(2, "mm"),
           just=c("left", "top"))
           
	# Flush output
	dev.off()
}
